@@ -183,8 +183,7 @@ parseTags <- function(bodySection, initCollection) {
 #'
 #' The \code{createModels} function processes a single Mplus template file and creates a group of related
 #' model input files. Definitions and examples for the template language are provided in the MplusAutomation
-#' vignette and are not duplicated here at the moment. See this PDF:
-#' \url{http://cran.r-project.org/web/packages/MplusAutomation/vignettes/Vignette.pdf}
+#' vignette and are not duplicated here at the moment. See this documentation: \code{vignette("Vignette", package="MplusAutomation")}
 #'
 #' @param templatefile The filename (absolute or relative path) of an Mplus template file to be processed. Example \dQuote{C:/MplusTemplate.txt}
 #' @return No value is returned by this function. It is solely used to process an Mplus template file.
@@ -1178,6 +1177,128 @@ prepareMplusData_Mat <- function(covMatrix, meansMatrix, nobs) {
 
 }
 
+#' Clean data and calculate the md5 hash
+#'
+#' Internal utility function, primarily for \code{prepareMplusData}.
+#'
+#' @param df The R data.frame to be prepared for Mplus
+#' @param keepCols A character vector specifying the variable names
+#'   within \code{df} to be output to \code{filename} or a numeric
+#'   vector of the column indices to be output or a logical vector
+#'   corresponding to the same.
+#' @param dropCols A character vector specifying the variable names
+#'   within \code{df} to be omitted from the data output to \code{filename}
+#'   or a numeric vector of the column indices not to be output
+#'   or a logical vector corresponding to the same.
+#' @param imputed A logical whether data are multiply imputed.  Defaults
+#'   to \code{FALSE}.  If \code{TRUE}, the data should be a list,
+#'   where each element of the list is a multiply imputed dataset.
+#' @return A list of the data and the md5 hash.
+#' @keywords internal
+#' @importFrom digest digest
+#' @rdname MplusAutomationUtils
+#' @examples
+#' \dontrun{
+#'   MplusAutomation:::.cleanHashData(mtcars)
+#' }
+.cleanHashData <- function(df, keepCols, dropCols, imputed=FALSE) {
+  if (imputed) {
+    stopifnot(inherits(df, "list"))
+  } else {
+    stopifnot(inherits(df, "data.frame"))
+  }
+
+  ## only allow keep OR drop.
+  if(!missing(keepCols) && !missing(dropCols) && length(keepCols) && length(dropCols)) {
+    stop("keepCols and dropCols passed. You must choose one or the other, but not both.")
+  }
+
+  ## assert types allowed for keep and drop cols
+  stopifnot(missing(keepCols) || is.character(keepCols) ||
+            is.numeric(keepCols) || is.logical(keepCols))
+
+  stopifnot(missing(dropCols) || is.character(dropCols) ||
+            is.numeric(dropCols) || is.logical(dropCols))
+
+
+  ## keep only columns specified by keepCols
+  if (!missing(keepCols) && length(keepCols) > 0) {
+    if (imputed) {
+      df <- lapply(df, function(d) d[, keepCols, drop = FALSE])
+    } else {
+      df <- df[, keepCols, drop = FALSE] # works with all types
+    }
+  }
+
+  ## drop columns specified by dropCols
+  if (!missing(dropCols) && length(dropCols) > 0) {
+    if (is.character(dropCols)) {
+      if (imputed) {
+        df <- lapply(df, function(d) {subset(d, select = -which(colnames(d) %in% dropCols))})
+      } else {
+        df <- subset(df, select = -which(colnames(df) %in% dropCols))
+      }
+    } else if (is.numeric(dropCols)) {
+      if (imputed) {
+        df <- lapply(df, function(d) {subset(d, select = -dropCols)})
+      } else {
+        df <- subset(df, select = -dropCols)
+      }
+    } else if (is.logical(dropCols)) {
+      if (imputed) {
+        df <- lapply(df, function(d) {subset(d, select = !dropCols)})
+      } else {
+        df <- subset(df, select = !dropCols)
+      }
+    }
+  }
+
+  f <- function(x) {
+    as.vector(c(
+    dim(x),
+    unlist(lapply(x, class)),
+    unlist(dimnames(x)),
+    as.character(unlist(x[c(1, nrow(x)), ]))))
+  }
+
+  if (imputed) {
+    hash <- digest(as.vector(unlist(lapply(df, f))), "md5")
+  } else {
+    hash <- digest(f(df), "md5")
+  }
+
+  return(list(data = df, md5 = hash))
+}
+
+#' Check if a file exists with a given hash and add a hash to an existing filename
+#'
+#' Internal utility function, primarily for \code{prepareMplusData}.
+#'
+#' @param filename A character vector containing the filename
+#' @param hash A character vector with the hash to use
+#' @param useexisting A logical whether to use an existing file name
+#'   if one is found containing the hash.  Defaults to \code{FALSE}
+#'   in which case the hash is added to the user specified filename
+#' @return A list of the filename (plus hash) and a logical value
+#'   whether a filename with the hash already existed or not.
+#' @keywords internal
+#' @rdname MplusAutomationUtils
+#' @examples
+#' MplusAutomation:::.hashifyFile("testit.dat", "abc")
+.hashifyFile <- function(filename, hash, useexisting = FALSE) {
+  fileonly <- basename(filename)
+  allfiles <- list.files(path = dirname(filename))
+  existingfile <- grep(hash, allfiles, value=TRUE)[1]
+
+  fileexists <- length(existingfile) && !isTRUE(is.na(existingfile))
+
+  if (fileexists && useexisting) {
+    filename <- gsub(basename(filename), basename(existingfile), filename)
+  } else {
+    filename <- gsub("\\.dat$", paste0("_", hash, ".dat"), filename)
+  }
+  list(filename = filename, fileexists = fileexists)
+}
 
 #' Create tab-delimited file and Mplus input syntax from R data.frame
 #'
@@ -1187,6 +1308,22 @@ prepareMplusData_Mat <- function(covMatrix, meansMatrix, nobs) {
 #' data file definition and variable names,
 #' is printed to the console or optionally to an input file.
 #'
+#' The \code{writeData} argument is new and can be used to reduce overhead
+#' from repeatedly writing the same data from R to the disk.  When using the
+#' \sQuote{always} option, \code{prepareMplusData} behaves as before, always writing
+#' data from R to the disk.  When \sQuote{ifmissing}, R generates an
+#' md5 hash of the data prior to writing it out to the disk.  The md5 hash is based on:
+#' (1) the dimensions of the dataset, (2) the variable names,
+#' (3) the class of every variable, and (4) the raw data from the first and last rows.
+#' This combination ensures that under most all circumstances, if the data changes,
+#' the hash will change.  The hash is appended to the specified data file name
+#' (which is controlled by the logical \code{hashfilename} argument).  Next R
+#' checks in the directory where the data would normally be written.  If a data file
+#' exists in that directory that matches the hash generated from the data, R will
+#' use that existing data file instead of writing out the data again.
+#' A final option is \sQuote{never}.  If this option is used, R will not write
+#' the data out even if no file matching the hash is found.
+#'
 #' @param df The R data.frame to be prepared for Mplus
 #' @param filename The path and filename for the tab-delimited data file
 #'   for use with Mplus. Example: "C:/Mplusdata/data1.dat"
@@ -1216,6 +1353,14 @@ prepareMplusData_Mat <- function(covMatrix, meansMatrix, nobs) {
 #' @param imputed A logical whether data are multiply imputed.  Defaults
 #'   to \code{FALSE}.  If \code{TRUE}, the data should be a list,
 #'   where each element of the list is a multiply imputed dataset.
+#' @param writeData A character vector, one of \sQuote{always},
+#'   \sQuote{ifmissing}, \sQuote{never} indicating whether the data files
+#'   (*.dat) should be written to disk.  Defaults to
+#'   \sQuote{always} for consistency with previous behavior.
+#'   See details for further information.
+#' @param hashfilename A logical whether or not to add a hash of the raw data to the
+#'   data file name.  Defaults to \code{FALSE} for consistency with previous
+#'   behavior where this feature was not available..
 #' @return Invisibly returns a character vector of the Mplus input
 #'   syntax. Primarily called for its side effect of creating Mplus
 #'   data files and optionally input files.
@@ -1239,6 +1384,20 @@ prepareMplusData_Mat <- function(covMatrix, meansMatrix, nobs) {
 #' # see that syntax was stored
 #' test01
 #'
+#' # by default, if re-run, data is re-written, with a note
+#' test01b <- prepareMplusData(mtcars, "test01.dat")
+#'
+#' # if we turn on hashing in the filename the first time,
+#' # we can avoid overwriting notes the second time
+#' test01c <- prepareMplusData(mtcars, "test01c.dat", hashfilename=TRUE)
+#'
+#' # now that the filename was hashed in test01c, future calls do not re-write data
+#' # as long as the hash matches
+#' test01d <- prepareMplusData(mtcars, "test01c.dat", writeData = "ifmissing", hashfilename=TRUE)
+#'
+#' # however, if the data change, then the file is re-written
+#' test01e <- prepareMplusData(iris, "test01c.dat", writeData = "ifmissing", hashfilename=TRUE)
+#'
 #' # tests for keeping and dropping variables
 #' prepareMplusData(mtcars, "test02.dat", keepCols = c("mpg", "hp"))
 #' prepareMplusData(mtcars, "test03.dat", keepCols = c(1, 2))
@@ -1277,27 +1436,51 @@ prepareMplusData_Mat <- function(covMatrix, meansMatrix, nobs) {
 #' prepareMplusData(mtcars, "test10.dat",
 #'   inpfile="test10alt.inp", overwrite=FALSE)
 #' }
-
 prepareMplusData <- function(df, filename, keepCols, dropCols, inpfile=FALSE,
-  interactive=TRUE, overwrite=TRUE, imputed=FALSE) {
+                             interactive=TRUE, overwrite=TRUE, imputed=FALSE,
+                             writeData=c("always", "ifmissing", "never"), hashfilename=FALSE) {
+
+  writeData <- match.arg(writeData)
+
+  ## message and then exit function if never write data
+  if (identical(writeData, "never")) {
+    message("No action taken as writeData = 'never'")
+    return(invisible(""))
+  }
 
   if (imputed) {
-      stopifnot(inherits(df, "list"))
+    if (identical(writeData, "ifmissing")) {
+      writeData <- "always"
+      message("When imputed = TRUE, writeData cannot be 'ifmissing', setting to 'always'")
+    }
+    if (hashfilename) {
+      hashfilename <- FALSE
+      message("When imputed = TRUE, hashfilename cannot be TRUE, setting to FALSE")
+    }
+  }
+
+  if (!hashfilename && identical(writeData, "ifmissing")) {
+    writeData <- "always"
+    message("When hashfilename = FALSE, writeData cannot be 'ifmissing', setting to 'always'")
+  }
+
+  if (missing(keepCols)) {
+    if (missing(dropCols)) {
+      cleand <- .cleanHashData(df = df, imputed = imputed)
+    } else {
+      cleand <- .cleanHashData(df = df, dropCols = dropCols, imputed = imputed)
+    }
   } else {
-      stopifnot(inherits(df, "data.frame"))
+    if (missing(dropCols)) {
+      cleand <- .cleanHashData(df = df, keepCols = keepCols, imputed = imputed)
+    } else {
+      cleand <- .cleanHashData(df = df, keepCols = keepCols, dropCols = dropCols, imputed = imputed)
+    }
   }
 
-  ## only allow keep OR drop.
-  if(!missing(keepCols) && !missing(dropCols)) {
-    stop("keepCols and dropCols passed to prepareMplusData. You must choose one or the other, but not both.")
-  }
-
-  ## assert types allowed for keep and drop cols
-  stopifnot(missing(keepCols) || is.character(keepCols) ||
-    is.numeric(keepCols) || is.logical(keepCols))
-
-  stopifnot(missing(dropCols) || is.character(dropCols) ||
-    is.numeric(dropCols) || is.logical(dropCols))
+  df <- cleand$data
+  md5 <- cleand$md5
+  rm(cleand)
 
   ## if filename is missing and interactive is TRUE
   ## interactively (through GUI or console)
@@ -1306,109 +1489,91 @@ prepareMplusData <- function(df, filename, keepCols, dropCols, inpfile=FALSE,
     filename <- file.choose()
   }
 
-  ## if filename is still missing at this point
-  ## throw an error
+  ## if filename is still missing at this point, throw an error
   stopifnot(!missing(filename))
 
-  ## keep only columns specified by keepCols
-  if (!missing(keepCols) && length(keepCols) > 0) {
+  tmp <- .hashifyFile(filename, md5, useexisting = identical(writeData, "ifmissing"))
+
+  if (hashfilename) {
+    filename <- tmp$filename
+  }
+
+  if (identical(writeData, "ifmissing") && tmp$fileexists) {
+    message(sprintf("File with md5 hash matching data found, using %s", filename))
+  } else {
+    ## even if writeData = 'ifmissing' if the data are missing, need to write out
+    writeData <- "always"
+  }
+
+  if (identical(writeData, "always")) {
+    ## convert factors to numbers
     if (imputed) {
-      df <- lapply(df, function(d) d[, keepCols])
-    } else {
-      df <- df[, keepCols] # works with all types
-    }
-  }
-
-  ## drop columns specified by dropCols
-  if (!missing(dropCols) && length(dropCols) > 0) {
-    if (is.character(dropCols)) {
-      if (imputed) {
-        df <- lapply(df, function(d) {subset(d, select = -which(colnames(d) %in% dropCols))})
-      } else {
-          df <- subset(df, select = -which(colnames(df) %in% dropCols))
-        }
-    } else if (is.numeric(dropCols)) {
-        if (imputed) {
-          df <- lapply(df, function(d) {subset(d, select = -dropCols)})
-        } else {
-            df <- subset(df, select = -dropCols)
-          }
-      } else if (is.logical(dropCols)) {
-          if (imputed) {
-            df <- lapply(df, function(d) {subset(d, select = !dropCols)})
-          } else {
-              df <- subset(df, select = !dropCols)
-            }
-        }
-  }
-
-  ## convert factors to numbers
-  if (imputed) {
-  df <- lapply(1:length(df), function(i) {
-     as.data.frame(qv <- lapply(1:ncol(df[[i]]), function(col) {
-            if (is.factor(df[[i]][, col])) {
-              ## numeric storage of the levels corresponds to the order of the levels
-              if (i == 1) {
-                cat("Factor variable:", names(df[[i]])[col], "; factor levels:", paste(levels(df[[i]][, col]), collapse=", "), "converted to numbers:",
-                    paste(seq_along(levels(df[[i]][, col])), collapse=", "), "\n\n")
+    df <- lapply(1:length(df), function(i) {
+       as.data.frame(qv <- lapply(1:ncol(df[[i]]), function(col) {
+              if (is.factor(df[[i]][, col])) {
+                ## numeric storage of the levels corresponds to the order of the levels
+                if (i == 1) {
+                  cat("Factor variable:", names(df[[i]])[col], "; factor levels:", paste(levels(df[[i]][, col]), collapse=", "), "converted to numbers:",
+                      paste(seq_along(levels(df[[i]][, col])), collapse=", "), "\n\n")
+                }
+                col_value <- as.numeric(df[[i]][, col])
+              } else {
+                col_value <- df[[i]][, col]
               }
-              col_value <- as.numeric(df[[i]][, col])
-            } else {
-              col_value <- df[[i]][, col]
-            }
 
-            if (is.character(df[[i]][, col])) {
-              if (i == 1) warning("Character data requested for output using prepareMplusData.\n  Mplus does not support character data.")
-            }
+              if (is.character(df[[i]][, col])) {
+                if (i == 1) warning("Character data requested for output using prepareMplusData.\n  Mplus does not support character data.")
+              }
 
-            col_value <- list(col_value)
-            names(col_value) <- names(df[[i]])[col]
-            return(col_value)
-          }))
-   })
-  } else {
-
-  df <- as.data.frame(qv <- lapply(1:ncol(df), function(col) {
-            #can't use ifelse because is.factor returns only one element,
-            #and ifelse enforces identical length
-            if (is.factor(df[, col])) {
-              # numeric storage of the levels corresponds to the order of the levels
-              message("Factor variable:", names(df)[col], "; factor levels:", paste(levels(df[,col]), collapse=", "), "converted to numbers:",
-                  paste(seq_along(levels(df[,col])), collapse=", "), "\n\n")
-              col_value <- as.numeric(df[,col])
-            } else {
-              col_value <- df[,col]
-            }
-
-            if (is.character(df[,col])) {
-              warning("Character data requested for output using prepareMplusData.\n  Mplus does not support character data.")
-            }
-
-            col_value <- list(col_value)
-            names(col_value) <- names(df)[col]
-            return(col_value)
-          }))
-  }
-
-  if (file.exists(filename)) {
-    if (overwrite) {
-      warning(paste("The file", sQuote(basename(filename)),
-        "currently exists and will be overwritten"))
+              col_value <- list(col_value)
+              names(col_value) <- names(df[[i]])[col]
+              return(col_value)
+            }))
+     })
     } else {
-      stop(paste("The file", sQuote(basename(filename)),
-        "currently exists. Specify a different filename or set overwrite=TRUE"))
-    }
-  }
 
-  if (imputed) {
-    filename.base <- gsub("\\.dat", "", filename)
-    junk <- lapply(1:length(df), function(i) {
-      write.table(df[[i]], paste0(filename.base, "_imp_", i, ".dat"), sep = "\t",
-        col.names = FALSE, row.names = FALSE, na=".")
-    })
-    cat(paste0(filename.base, "_imp_", 1:length(df), ".dat"), file = filename, sep = "\n")
-  } else {
-    write.table(df, filename, sep = "\t", col.names = FALSE, row.names = FALSE, na=".")
+    df <- as.data.frame(qv <- lapply(1:ncol(df), function(col) {
+              #can't use ifelse because is.factor returns only one element,
+              #and ifelse enforces identical length
+              if (is.factor(df[, col])) {
+                # numeric storage of the levels corresponds to the order of the levels
+                message("Factor variable:", names(df)[col], "; factor levels:", paste(levels(df[,col]), collapse=", "), "converted to numbers:",
+                    paste(seq_along(levels(df[,col])), collapse=", "), "\n\n")
+                col_value <- as.numeric(df[,col])
+              } else {
+                col_value <- df[,col]
+              }
+
+              if (is.character(df[,col])) {
+                warning("Character data requested for output using prepareMplusData.\n  Mplus does not support character data.")
+              }
+
+              col_value <- list(col_value)
+              names(col_value) <- names(df)[col]
+              return(col_value)
+            }))
+    }
+
+    if (file.exists(filename)) {
+      if (overwrite) {
+        warning(paste("The file", sQuote(basename(filename)),
+          "currently exists and will be overwritten"))
+      } else {
+        stop(paste("The file", sQuote(basename(filename)),
+          "currently exists. Specify a different filename or set overwrite=TRUE"))
+      }
+    }
+
+    if (imputed) {
+      filename.base <- gsub("\\.dat", "", filename)
+      junk <- lapply(1:length(df), function(i) {
+        write.table(df[[i]], paste0(filename.base, "_imp_", i, ".dat"), sep = "\t",
+          col.names = FALSE, row.names = FALSE, na=".")
+      })
+      cat(paste0(filename.base, "_imp_", 1:length(df), ".dat"), file = filename, sep = "\n")
+    } else {
+      write.table(df, filename, sep = "\t", col.names = FALSE, row.names = FALSE, na=".")
+    }
   }
 
   if (imputed) {
@@ -1452,6 +1617,6 @@ prepareMplusData <- function(df, filename, keepCols, dropCols, inpfile=FALSE,
   # write out syntax, either to stdout or to a file
   cat(syntax, file=inpfile, sep="")
 
-  # return invisible so it can be saved/reused if desired
+  # return syntax invisibly for later use/reuse
   return(invisible(syntax))
 }
